@@ -1,12 +1,7 @@
-/*
- * bacula.h -- main header file to include in all Bacula source
- *
- *   Version $Id: bacula.h 4992 2007-06-07 14:46:43Z kerns $
- */
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2000-2006 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2009 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -25,15 +20,27 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 
-   Bacula® is a registered trademark of John Walker.
+   Bacula® is a registered trademark of Kern Sibbald.
    The licensor of Bacula is the Free Software Foundation Europe
    (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
    Switzerland, email:ftf@fsfeurope.org.
 */
+/*
+ * bacula.h -- main header file to include in all Bacula source
+ *
+ *   Version $Id$
+ */
 
 #ifndef _BACULA_H
 #define _BACULA_H 1
 
+/* Disable FORTIFY_SOURCE, because bacula uses is own memory
+ * manager
+ */
+#ifdef _FORTIFY_SOURCE
+#undef _FORTIFY_SOURCE
+#endif
+
 #ifdef __cplusplus
 /* Workaround for SGI IRIX 6.5 */
 #define _LANGUAGE_C_PLUS_PLUS 1
@@ -48,6 +55,7 @@
 #else
 #include "config.h"
 #endif
+#define __CONFIG_H
 
 
 #define _REENTRANT    1
@@ -57,8 +65,10 @@
 
 /* System includes */
 #if HAVE_STDINT_H
+#ifndef __sgi
 #include <stdint.h>
 #endif
+#endif
 #if HAVE_STDARG_H
 #include <stdarg.h>
 #endif
