@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2000-2007 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2009 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -20,7 +20,7 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 
-   Bacula® is a registered trademark of John Walker.
+   Bacula® is a registered trademark of Kern Sibbald.
    The licensor of Bacula is the Free Software Foundation Europe
    (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
    Switzerland, email:ftf@fsfeurope.org.
@@ -38,7 +38,7 @@
  *       to do the backup.
  *     When the File daemon finishes the job, update the DB.
  *
- *   Version $Id: backup.c 6260 2008-01-09 07:29:57Z junkmale $
+ *   Version $Id$
  */
 
 #include "bacula.h"
@@ -65,6 +65,9 @@ static char OldEndJob[]  = "2800 End Job TermCode=%d JobFiles=%u "
 bool do_backup_init(JCR *jcr)
 {
 
+   if (jcr->getJobLevel() == L_VIRTUAL_FULL) {
+      return do_vbackup_init(jcr);
+   }
    free_rstorage(jcr);                   /* we don't read so release */
 
    if (!get_or_create_fileset_record(jcr)) {
@@ -78,6 +81,10 @@ bool do_backup_init(JCR *jcr)
 
    apply_pool_overrides(jcr);
 
+   if (!allow_duplicate_job(jcr)) {
+      return false;
+   }
+
    jcr->jr.PoolId = get_or_create_pool_record(jcr, jcr->pool->name());
    if (jcr->jr.PoolId == 0) {
       return false;
@@ -96,6 +103,209 @@ bool do_backup_init(JCR *jcr)
    return true;
 }
 
+/* Take all base jobs from job resource and find the
+ * last L_BASE jobid.
+ */
+static bool get_base_jobids(JCR *jcr, db_list_ctx *jobids)
+{
+   JOB_DBR jr;
+   JOB *job;
+   JobId_t id;
+   char str_jobid[50];
+
+   if (!jcr->job->base) {
+      return false;             /* no base job, stop accurate */
+   }
+
+   memset(&jr, 0, sizeof(JOB_DBR));
+   jr.StartTime = jcr->jr.StartTime;
+
+   foreach_alist(job, jcr->job->base) {
+      bstrncpy(jr.Name, job->name(), sizeof(jr.Name));
+      db_get_base_jobid(jcr, jcr->db, &jr, &id);
+
+      if (id) {
+         if (jobids->count) {
+            pm_strcat(jobids->list, ",");
+         }
+         pm_strcat(jobids->list, edit_uint64(id, str_jobid));
+         jobids->count++;
+      }
+   }
+
+   return jobids->count > 0;
+}
+
+/*
+ * Foreach files in currrent list, send "/path/fname\0LStat\0MD5" to FD
+ */
+static int accurate_list_handler(void *ctx, int num_fields, char **row)
+{
+   JCR *jcr = (JCR *)ctx;
+
+   if (job_canceled(jcr)) {
+      return 1;
+   }
+   
+   if (row[2] == 0) {           /* discard when file_index == 0 */
+      return 0;
+   }
+
+   /* sending with checksum */
+   if (jcr->use_accurate_chksum 
+       && num_fields == 6 
+       && row[5][0] /* skip checksum = '0' */
+       && row[5][1])
+   { 
+      jcr->file_bsock->fsend("%s%s%c%s%c%s", 
+                             row[0], row[1], 0, row[4], 0, row[5]); 
+   } else {
+      jcr->file_bsock->fsend("%s%s%c%s", 
+                             row[0], row[1], 0, row[4]); 
+   }
+   return 0;
+}
+
+/* In this procedure, we check if the current fileset is using checksum
+ * FileSet-> Include-> Options-> Accurate/Verify/BaseJob=checksum
+ * This procedure uses jcr->HasBase, so it must be call after the initialization
+ */
+static bool is_checksum_needed_by_fileset(JCR *jcr)
+{
+   FILESET *f;
+   INCEXE *inc;
+   FOPTS *fopts;
+   bool in_block=false;
+   bool have_basejob_option=false;
+   if (!jcr->job || !jcr->job->fileset) {
+      return false;
+   }
+
+   f = jcr->job->fileset;
+   
+   for (int i=0; i < f->num_includes; i++) { /* Parse all Include {} */
+      inc = f->include_items[i];
+      
+      for (int j=0; j < inc->num_opts; j++) { /* Parse all Options {} */
+         fopts = inc->opts_list[j];
+         
+         for (char *k=fopts->opts; *k ; k++) { /* Try to find one request */
+            switch (*k) {
+            case 'V':           /* verify */
+               in_block = (jcr->getJobType() == JT_VERIFY); /* not used now */
+               break;
+            case 'J':           /* Basejob keyword */
+               have_basejob_option = in_block = jcr->HasBase;
+               break;
+            case 'C':           /* Accurate keyword */
+               in_block = (jcr->getJobLevel() != L_FULL);
+               break;
+            case ':':           /* End of keyword */
+               in_block = false;
+               break;
+            case '5':           /* MD5  */
+            case '1':           /* SHA1 */
+               if (in_block) {
+                  Dmsg0(50, "Checksum will be sent to FD\n");
+                  return true;
+               }
+               break;
+            default:
+               break;
+            }
+         }
+      }
+   }
+
+   /* By default for BaseJobs, we send the checksum */
+   if (!have_basejob_option && jcr->HasBase) {
+      return true;
+   }
+   
+   Dmsg0(50, "Checksum will be sent to FD\n");
+   return false;
+}
+
+/*
+ * Send current file list to FD
+ *    DIR -> FD : accurate files=xxxx
+ *    DIR -> FD : /path/to/file\0Lstat\0MD5
+ *    DIR -> FD : /path/to/dir/\0Lstat\0MD5
+ *    ...
+ *    DIR -> FD : EOD
+ */
+bool send_accurate_current_files(JCR *jcr)
+{
+   POOL_MEM buf;
+   bool ret=true;
+   db_list_ctx jobids;
+   db_list_ctx nb;
+
+   if (!jcr->accurate || job_canceled(jcr)) {
+      return true;
+   }
+   /* In base level, no previous job is used */
+   if (jcr->getJobLevel() == L_BASE) {
+      return true;
+   }
+   
+   if (jcr->getJobLevel() == L_FULL) {
+      /* On Full mode, if no previous base job, no accurate things */
+      if (!get_base_jobids(jcr, &jobids)) {
+         goto bail_out;
+      }
+      jcr->HasBase = true;
+      Jmsg(jcr, M_INFO, 0, _("Using BaseJobId(s): %s\n"), jobids.list);
+
+   } else {
+      /* For Incr/Diff level, we search for older jobs */
+      db_accurate_get_jobids(jcr, jcr->db, &jcr->jr, &jobids);
+
+      /* We are in Incr/Diff, but no Full to build the accurate list... */
+      if (jobids.count == 0) {
+         ret=false;
+         Jmsg(jcr, M_FATAL, 0, _("Cannot find previous jobids.\n"));
+         goto bail_out;
+      }
+   }
+
+   /* Don't send and store the checksum if fileset doesn't require it */
+   jcr->use_accurate_chksum = is_checksum_needed_by_fileset(jcr);
+
+   if (jcr->JobId) {            /* display the message only for real jobs */
+      Jmsg(jcr, M_INFO, 0, _("Sending Accurate information.\n"));
+   }
+
+   /* to be able to allocate the right size for htable */
+   Mmsg(buf, "SELECT sum(JobFiles) FROM Job WHERE JobId IN (%s)", jobids.list);
+   db_sql_query(jcr->db, buf.c_str(), db_list_handler, &nb);
+   Dmsg2(200, "jobids=%s nb=%s\n", jobids.list, nb.list);
+   jcr->file_bsock->fsend("accurate files=%s\n", nb.list); 
+
+   if (!db_open_batch_connexion(jcr, jcr->db)) {
+      Jmsg0(jcr, M_FATAL, 0, "Can't get batch sql connexion");
+      return false;
+   }
+   
+   if (jcr->HasBase) {
+      jcr->nb_base_files = str_to_int64(nb.list);
+      db_create_base_file_list(jcr, jcr->db, jobids.list);
+      db_get_base_file_list(jcr, jcr->db, 
+                            accurate_list_handler, (void *)jcr);
+
+   } else {
+      db_get_file_list(jcr, jcr->db_batch, jobids.list, 
+                       accurate_list_handler, (void *)jcr);
+   } 
+
+   /* TODO: close the batch connexion ? (can be used very soon) */
+
+   jcr->file_bsock->signal(BNET_EOD);
+
+bail_out:
+   return ret;
+}
+
 /*
  * Do a backup of the specified FileSet
  *
@@ -110,12 +320,15 @@ bool do_backup(JCR *jcr)
    STORE *store;
    char ed1[100];
 
+   if (jcr->getJobLevel() == L_VIRTUAL_FULL) {
+      return do_vbackup(jcr);
+   }
 
    /* Print Job Start message */
    Jmsg(jcr, M_INFO, 0, _("Start Backup JobId %s, Job=%s\n"),
         edit_uint64(jcr->JobId, ed1), jcr->Job);
 
-   set_jcr_job_status(jcr, JS_Running);
+   jcr->setJobStatus(JS_Running);
    Dmsg2(100, "JobId=%d JobLevel=%c\n", jcr->jr.JobId, jcr->jr.JobLevel);
    if (!db_update_job_start_record(jcr, jcr->db, &jcr->jr)) {
       Jmsg(jcr, M_FATAL, 0, "%s", db_strerror(jcr->db));
@@ -129,7 +342,7 @@ bool do_backup(JCR *jcr)
     *
     */
    Dmsg0(110, "Open connection with storage daemon\n");
-   set_jcr_job_status(jcr, JS_WaitSD);
+   jcr->setJobStatus(JS_WaitSD);
    /*
     * Start conversation with Storage daemon
     */
@@ -163,12 +376,12 @@ bool do_backup(JCR *jcr)
    }
    Dmsg0(150, "Storage daemon connection OK\n");
 
-   set_jcr_job_status(jcr, JS_WaitFD);
+   jcr->setJobStatus(JS_WaitFD);
    if (!connect_to_file_daemon(jcr, 10, FDConnectTimeout, 1)) {
       goto bail_out;
    }
 
-   set_jcr_job_status(jcr, JS_Running);
+   jcr->setJobStatus(JS_Running);
    fd = jcr->file_bsock;
 
    if (!send_include_list(jcr)) {
@@ -225,6 +438,14 @@ bool do_backup(JCR *jcr)
       Jmsg(jcr, M_FATAL, 0, "%s", db_strerror(jcr->db));
    }
 
+   /*
+    * If backup is in accurate mode, we send the list of
+    * all files to FD.
+    */
+   if (!send_accurate_current_files(jcr)) {
+      goto bail_out;
+   }
+
    /* Send backup command */
    fd->fsend(backupcmd);
    if (!response(jcr, fd, OKbackup, "backup", DISPLAY_ERROR)) {
@@ -234,6 +455,13 @@ bool do_backup(JCR *jcr)
    /* Pickup Job termination data */
    stat = wait_for_job_termination(jcr);
    db_write_batch_file_records(jcr);    /* used by bulk batch file insert */
+
+   if (jcr->HasBase && 
+       !db_commit_base_file_attributes_record(jcr, jcr->db)) 
+   {
+         Jmsg(jcr, M_FATAL, 0, "%s", db_strerror(jcr->db));
+   }
+
    if (stat == JS_Terminated) {
       backup_cleanup(jcr, stat);
       return true;
@@ -242,11 +470,10 @@ bool do_backup(JCR *jcr)
 
 /* Come here only after starting SD thread */
 bail_out:
-   set_jcr_job_status(jcr, JS_ErrorTerminated);
+   jcr->setJobStatus(JS_ErrorTerminated);
    Dmsg1(400, "wait for sd. use=%d\n", jcr->use_count());
    /* Cancel SD */
-   cancel_storage_daemon_job(jcr);
-   wait_for_storage_daemon_termination(jcr);
+   wait_for_job_termination(jcr, FDConnectTimeout);
    Dmsg1(400, "after wait for sd. use=%d\n", jcr->use_count());
    return false;
 }
@@ -258,58 +485,69 @@ bail_out:
  *   are done, we return the job status.
  * Also used by restore.c
  */
-int wait_for_job_termination(JCR *jcr)
+int wait_for_job_termination(JCR *jcr, int timeout)
 {
    int32_t n = 0;
    BSOCK *fd = jcr->file_bsock;
    bool fd_ok = false;
-   uint32_t JobFiles, Errors;
+   uint32_t JobFiles, JobErrors;
+   uint32_t JobWarnings = 0;
    uint64_t ReadBytes = 0;
    uint64_t JobBytes = 0;
    int VSS = 0;
    int Encrypt = 0;
+   btimer_t *tid=NULL;
 
-   set_jcr_job_status(jcr, JS_Running);
-   /* Wait for Client to terminate */
-   while ((n = bget_dirmsg(fd)) >= 0) {
-      if (!fd_ok && 
-          (sscanf(fd->msg, EndJob, &jcr->FDJobStatus, &JobFiles,
-              &ReadBytes, &JobBytes, &Errors, &VSS, &Encrypt) == 7 ||
-           sscanf(fd->msg, OldEndJob, &jcr->FDJobStatus, &JobFiles,
-                 &ReadBytes, &JobBytes, &Errors) == 5)) {
-         fd_ok = true;
-         set_jcr_job_status(jcr, jcr->FDJobStatus);
-         Dmsg1(100, "FDStatus=%c\n", (char)jcr->JobStatus);
-      } else {
-         Jmsg(jcr, M_WARNING, 0, _("Unexpected Client Job message: %s\n"),
-            fd->msg);
-      }
-      if (job_canceled(jcr)) {
-         break;
-      }
-   }
+   jcr->setJobStatus(JS_Running);
 
-   if (is_bnet_error(fd)) {
-      Jmsg(jcr, M_FATAL, 0, _("Network error with FD during %s: ERR=%s\n"),
-          job_type_to_str(jcr->JobType), fd->bstrerror());
+   if (fd) {
+      if (timeout) {
+         tid = start_bsock_timer(fd, timeout); /* TODO: New timeout directive??? */
+      }
+      /* Wait for Client to terminate */
+      while ((n = bget_dirmsg(fd)) >= 0) {
+         if (!fd_ok && 
+             (sscanf(fd->msg, EndJob, &jcr->FDJobStatus, &JobFiles,
+                     &ReadBytes, &JobBytes, &JobErrors, &VSS, &Encrypt) == 7 ||
+              sscanf(fd->msg, OldEndJob, &jcr->FDJobStatus, &JobFiles,
+                     &ReadBytes, &JobBytes, &JobErrors) == 5)) {
+            fd_ok = true;
+            jcr->setJobStatus(jcr->FDJobStatus);
+            Dmsg1(100, "FDStatus=%c\n", (char)jcr->JobStatus);
+         } else {
+            Jmsg(jcr, M_WARNING, 0, _("Unexpected Client Job message: %s\n"),
+                 fd->msg);
+         }
+         if (job_canceled(jcr)) {
+            break;
+         }
+      }
+      if (tid) {
+         stop_bsock_timer(tid);
+      }
+
+      if (is_bnet_error(fd)) {
+         Jmsg(jcr, M_FATAL, 0, _("Network error with FD during %s: ERR=%s\n"),
+              job_type_to_str(jcr->getJobType()), fd->bstrerror());
+      }
+      fd->signal(BNET_TERMINATE);   /* tell Client we are terminating */
    }
-   fd->signal(BNET_TERMINATE);   /* tell Client we are terminating */
 
    /* Force cancel in SD if failing */
    if (job_canceled(jcr) || !fd_ok) {
       cancel_storage_daemon_job(jcr);
    }
 
-   /* Note, the SD stores in jcr->JobFiles/ReadBytes/JobBytes/Errors */
+   /* Note, the SD stores in jcr->JobFiles/ReadBytes/JobBytes/JobErrors */
    wait_for_storage_daemon_termination(jcr);
 
-
    /* Return values from FD */
    if (fd_ok) {
       jcr->JobFiles = JobFiles;
-      jcr->Errors = Errors;
+      jcr->JobErrors += JobErrors;       /* Keep total errors */
       jcr->ReadBytes = ReadBytes;
       jcr->JobBytes = JobBytes;
+      jcr->JobWarnings = JobWarnings;
       jcr->VSS = VSS;
       jcr->Encrypt = Encrypt;
    } else {
@@ -320,7 +558,7 @@ int wait_for_job_termination(JCR *jcr)
 //   jcr->JobStatus, jcr->SDJobStatus);
 
    /* Return the first error status we find Dir, FD, or SD */
-   if (!fd_ok || is_bnet_error(fd)) {
+   if (!fd_ok || is_bnet_error(fd)) { /* if fd not set, that use !fd_ok */
       jcr->FDJobStatus = JS_ErrorTerminated;
    }
    if (jcr->JobStatus != JS_Terminated) {
@@ -347,17 +585,30 @@ void backup_cleanup(JCR *jcr, int TermCode)
    CLIENT_DBR cr;
    double kbps, compression;
    utime_t RunTime;
+   POOL_MEM base_info;
+
+   if (jcr->getJobLevel() == L_VIRTUAL_FULL) {
+      vbackup_cleanup(jcr, TermCode);
+      return;
+   }
 
    Dmsg2(100, "Enter backup_cleanup %d %c\n", TermCode, TermCode);
    memset(&mr, 0, sizeof(mr));
    memset(&cr, 0, sizeof(cr));
 
+#ifdef xxxx
+   if (jcr->getJobStatus() == JS_Terminated && 
+        (jcr->JobErrors || jcr->SDErrors || jcr->JobWarnings)) {
+      TermCode = JS_Warnings;
+   }
+#endif
+         
    update_job_end(jcr, TermCode);
 
    if (!db_get_job_record(jcr, jcr->db, &jcr->jr)) {
       Jmsg(jcr, M_WARNING, 0, _("Error getting Job record for Job report: ERR=%s"),
          db_strerror(jcr->db));
-      set_jcr_job_status(jcr, JS_ErrorTerminated);
+      jcr->setJobStatus(JS_ErrorTerminated);
    }
 
    bstrncpy(cr.Name, jcr->client->name(), sizeof(cr.Name));
@@ -370,19 +621,22 @@ void backup_cleanup(JCR *jcr, int TermCode)
    if (!db_get_media_record(jcr, jcr->db, &mr)) {
       Jmsg(jcr, M_WARNING, 0, _("Error getting Media record for Volume \"%s\": ERR=%s"),
          mr.VolumeName, db_strerror(jcr->db));
-      set_jcr_job_status(jcr, JS_ErrorTerminated);
+      jcr->setJobStatus(JS_ErrorTerminated);
    }
 
    update_bootstrap_file(jcr);
 
    switch (jcr->JobStatus) {
       case JS_Terminated:
-         if (jcr->Errors || jcr->SDErrors) {
+         if (jcr->JobErrors || jcr->SDErrors) {
             term_msg = _("Backup OK -- with warnings");
          } else {
             term_msg = _("Backup OK");
          }
          break;
+      case JS_Warnings:
+         term_msg = _("Backup OK -- with warnings");
+         break;
       case JS_FatalError:
       case JS_ErrorTerminated:
          term_msg = _("*** Backup Error ***");
@@ -415,7 +669,7 @@ void backup_cleanup(JCR *jcr, int TermCode)
    if (RunTime <= 0) {
       kbps = 0;
    } else {
-      kbps = (double)jcr->jr.JobBytes / (1000 * RunTime);
+      kbps = ((double)jcr->jr.JobBytes) / (1000.0 * (double)RunTime);
    }
    if (!db_get_job_volume_names(jcr, jcr->db, jcr->jr.JobId, &jcr->VolumeName)) {
       /*
@@ -437,15 +691,21 @@ void backup_cleanup(JCR *jcr, int TermCode)
       if (compression < 0.5) {
          bstrncpy(compress, "None", sizeof(compress));
       } else {
-         bsnprintf(compress, sizeof(compress), "%.1f %%", (float)compression);
+         bsnprintf(compress, sizeof(compress), "%.1f %%", compression);
       }
    }
    jobstatus_to_ascii(jcr->FDJobStatus, fd_term_msg, sizeof(fd_term_msg));
    jobstatus_to_ascii(jcr->SDJobStatus, sd_term_msg, sizeof(sd_term_msg));
 
+   if (jcr->HasBase) {
+      Mmsg(base_info, "  Base files/Used files:  %lld/%lld (%.2f%%)\n",
+           jcr->nb_base_files, 
+           jcr->nb_base_files_used, 
+           jcr->nb_base_files_used*100.0/jcr->nb_base_files);
+   }
 // bmicrosleep(15, 0);                /* for debugging SIGHUP */
 
-   Jmsg(jcr, msg_type, 0, _("Bacula %s %s (%s): %s\n"
+   Jmsg(jcr, msg_type, 0, _("%s %s %s (%s): %s\n"
 "  Build OS:               %s %s %s\n"
 "  JobId:                  %d\n"
 "  Job:                    %s\n"
@@ -453,6 +713,7 @@ void backup_cleanup(JCR *jcr, int TermCode)
 "  Client:                 \"%s\" %s\n"
 "  FileSet:                \"%s\" %s\n"
 "  Pool:                   \"%s\" (From %s)\n"
+"  Catalog:                \"%s\" (From %s)\n"
 "  Storage:                \"%s\" (From %s)\n"
 "  Scheduled time:         %s\n"
 "  Start time:             %s\n"
@@ -465,8 +726,10 @@ void backup_cleanup(JCR *jcr, int TermCode)
 "  SD Bytes Written:       %s (%sB)\n"
 "  Rate:                   %.1f KB/s\n"
 "  Software Compression:   %s\n"
+"%s"                                         /* Basefile info */
 "  VSS:                    %s\n"
-"  Storage Encryption:     %s\n"
+"  Encryption:             %s\n"
+"  Accurate:               %s\n"
 "  Volume name(s):         %s\n"
 "  Volume Session Id:      %d\n"
 "  Volume Session Time:    %d\n"
@@ -476,14 +739,15 @@ void backup_cleanup(JCR *jcr, int TermCode)
 "  FD termination status:  %s\n"
 "  SD termination status:  %s\n"
 "  Termination:            %s\n\n"),
-        my_name, VERSION, LSMDATE, edt,
+        BACULA, my_name, VERSION, LSMDATE, edt,
         HOST_OS, DISTNAME, DISTVER,
         jcr->jr.JobId,
         jcr->jr.Job,
-        level_to_str(jcr->JobLevel), jcr->since,
+        level_to_str(jcr->getJobLevel()), jcr->since,
         jcr->client->name(), cr.Uname,
         jcr->fileset->name(), jcr->FSCreateTime,
         jcr->pool->name(), jcr->pool_source,
+        jcr->catalog->name(), jcr->catalog_source,
         jcr->wstore->name(), jcr->wstore_source,
         schedt,
         sdt,
@@ -496,16 +760,18 @@ void backup_cleanup(JCR *jcr, int TermCode)
         edit_uint64_with_suffix(jcr->jr.JobBytes, ec4),
         edit_uint64_with_commas(jcr->SDJobBytes, ec5),
         edit_uint64_with_suffix(jcr->SDJobBytes, ec6),
-        (float)kbps,
+        kbps,
         compress,
-        jcr->VSS?"yes":"no",
-        jcr->Encrypt?"yes":"no",
+        base_info.c_str(),
+        jcr->VSS?_("yes"):_("no"),
+        jcr->Encrypt?_("yes"):_("no"),
+        jcr->accurate?_("yes"):_("no"),
         jcr->VolumeName,
         jcr->VolSessionId,
         jcr->VolSessionTime,
         edit_uint64_with_commas(mr.VolBytes, ec7),
         edit_uint64_with_suffix(mr.VolBytes, ec8),
-        jcr->Errors,
+        jcr->JobErrors,
         jcr->SDErrors,
         fd_term_msg,
         sd_term_msg,
@@ -527,7 +793,7 @@ void update_bootstrap_file(JCR *jcr)
 
       VOL_PARAMS *VolParams = NULL;
       int VolCount;
-      char edt[50];
+      char edt[50], ed1[50], ed2[50];
 
       if (*fname == '|') {
          got_pipe = 1;
@@ -535,7 +801,7 @@ void update_bootstrap_file(JCR *jcr)
          fd = bpipe ? bpipe->wfd : NULL;
       } else {
          /* ***FIXME*** handle BASE */
-         fd = fopen(fname, jcr->JobLevel==L_FULL?"w+b":"a+b");
+         fd = fopen(fname, jcr->getJobLevel()==L_FULL?"w+b":"a+b");
       }
       if (fd) {
          VolCount = db_get_job_volume_parameters(jcr, jcr->db, jcr->JobId,
@@ -544,24 +810,26 @@ void update_bootstrap_file(JCR *jcr)
             Jmsg(jcr, M_ERROR, 0, _("Could not get Job Volume Parameters to "
                  "update Bootstrap file. ERR=%s\n"), db_strerror(jcr->db));
              if (jcr->SDJobFiles != 0) {
-                set_jcr_job_status(jcr, JS_ErrorTerminated);
+                jcr->setJobStatus(JS_ErrorTerminated);
              }
 
          }
          /* Start output with when and who wrote it */
          bstrftimes(edt, sizeof(edt), time(NULL));
          fprintf(fd, "# %s - %s - %s%s\n", edt, jcr->jr.Job,
-                 level_to_str(jcr->JobLevel), jcr->since);
+                 level_to_str(jcr->getJobLevel()), jcr->since);
          for (int i=0; i < VolCount; i++) {
             /* Write the record */
             fprintf(fd, "Volume=\"%s\"\n", VolParams[i].VolumeName);
             fprintf(fd, "MediaType=\"%s\"\n", VolParams[i].MediaType);
+            if (VolParams[i].Slot > 0) {
+               fprintf(fd, "Slot=%d\n", VolParams[i].Slot);
+            }
             fprintf(fd, "VolSessionId=%u\n", jcr->VolSessionId);
             fprintf(fd, "VolSessionTime=%u\n", jcr->VolSessionTime);
-            fprintf(fd, "VolFile=%u-%u\n", VolParams[i].StartFile,
-                         VolParams[i].EndFile);
-            fprintf(fd, "VolBlock=%u-%u\n", VolParams[i].StartBlock,
-                         VolParams[i].EndBlock);
+            fprintf(fd, "VolAddr=%s-%s\n", 
+                    edit_uint64(VolParams[i].StartAddr, ed1),
+                    edit_uint64(VolParams[i].EndAddr, ed2));
             fprintf(fd, "FileIndex=%d-%d\n", VolParams[i].FirstIndex,
                          VolParams[i].LastIndex);
          }
@@ -577,7 +845,7 @@ void update_bootstrap_file(JCR *jcr)
          berrno be;
          Jmsg(jcr, M_ERROR, 0, _("Could not open WriteBootstrap file:\n"
               "%s: ERR=%s\n"), fname, be.bstrerror());
-         set_jcr_job_status(jcr, JS_ErrorTerminated);
+         jcr->setJobStatus(JS_ErrorTerminated);
       }
       free_pool_memory(fname);
    }
