@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2002-2007 Free Software Foundation Europe e.V.
+   Copyright (C) 2002-2009 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -20,7 +20,7 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 
-   Bacula® is a registered trademark of John Walker.
+   Bacula® is a registered trademark of Kern Sibbald.
    The licensor of Bacula is the Free Software Foundation Europe
    (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
    Switzerland, email:ftf@fsfeurope.org.
@@ -32,7 +32,6 @@
  *
  *     Kern Sibbald, February MMII
  *
- *   Version $Id: ua_prune.c 8105 2008-12-02 18:49:00Z ricozz $
  */
 
 #include "bacula.h"
@@ -104,21 +103,26 @@ int file_delete_handler(void *ctx, int num_fields, char **row)
 /*
  *   Prune records from database
  *
- *    prune files (from) client=xxx
- *    prune jobs (from) client=xxx
+ *    prune files (from) client=xxx [pool=yyy]
+ *    prune jobs (from) client=xxx [pool=yyy]
  *    prune volume=xxx
+ *    prune stats
  */
 int prunecmd(UAContext *ua, const char *cmd)
 {
+   DIRRES *dir;
    CLIENT *client;
+   POOL *pool;
    POOL_DBR pr;
    MEDIA_DBR mr;
+   utime_t retention;
    int kw;
 
    static const char *keywords[] = {
       NT_("Files"),
       NT_("Jobs"),
       NT_("Volume"),
+      NT_("Stats"),
       NULL};
 
    if (!open_client_db(ua)) {
@@ -127,7 +131,7 @@ int prunecmd(UAContext *ua, const char *cmd)
 
    /* First search args */
    kw = find_arg_keyword(ua, keywords);
-   if (kw < 0 || kw > 2) {
+   if (kw < 0 || kw > 3) {
       /* no args, so ask user */
       kw = do_keyword_prompt(ua, _("Choose item to prune"), keywords);
    }
@@ -135,18 +139,38 @@ int prunecmd(UAContext *ua, const char *cmd)
    switch (kw) {
    case 0:  /* prune files */
       client = get_client_resource(ua);
-      if (!client || !confirm_retention(ua, &client->FileRetention, "File")) {
+      if (find_arg_with_value(ua, "pool") >= 0) {
+         pool = get_pool_resource(ua);
+      } else {
+         pool = NULL;
+      }
+      /* Pool File Retention takes precedence over client File Retention */
+      if (pool && pool->FileRetention > 0) {
+         if (!confirm_retention(ua, &pool->FileRetention, "File")) {
+            return false;
+         }
+      } else if (!client || !confirm_retention(ua, &client->FileRetention, "File")) {
          return false;
       }
-      prune_files(ua, client);
+      prune_files(ua, client, pool);
       return true;
    case 1:  /* prune jobs */
       client = get_client_resource(ua);
-      if (!client || !confirm_retention(ua, &client->JobRetention, "Job")) {
+      if (find_arg_with_value(ua, "pool") >= 0) {
+         pool = get_pool_resource(ua);
+      } else {
+         pool = NULL;
+      }
+      /* Pool Job Retention takes precedence over client Job Retention */
+      if (pool && pool->JobRetention > 0) {
+         if (!confirm_retention(ua, &pool->JobRetention, "Job")) {
+            return false;
+         }
+      } else if (!client || !confirm_retention(ua, &client->JobRetention, "Job")) {
          return false;
       }
       /* ****FIXME**** allow user to select JobType */
-      prune_jobs(ua, client, JT_BACKUP);
+      prune_jobs(ua, client, pool, JT_BACKUP);
       return 1;
    case 2:  /* prune volume */
       if (!select_pool_and_media_dbr(ua, &pr, &mr)) {
@@ -162,6 +186,17 @@ int prunecmd(UAContext *ua, const char *cmd)
       }
       prune_volume(ua, &mr);
       return true;
+   case 3:  /* prune stats */
+      dir = (DIRRES *)GetNextRes(R_DIRECTOR, NULL);
+      if (!dir->stats_retention) {
+         return false;
+      }
+      retention = dir->stats_retention;
+      if (!confirm_retention(ua, &retention, "Statistics")) {
+         return false;
+      }
+      prune_stats(ua, retention);
+      return true;
    default:
       break;
    }
@@ -169,6 +204,26 @@ int prunecmd(UAContext *ua, const char *cmd)
    return true;
 }
 
+/* Prune Job stat records from the database. 
+ *
+ */
+int prune_stats(UAContext *ua, utime_t retention)
+{
+   char ed1[50];
+   POOL_MEM query(PM_MESSAGE);
+   utime_t now = (utime_t)time(NULL);
+
+   db_lock(ua->db);
+   Mmsg(query, "DELETE FROM JobHisto WHERE JobTDate < %s", 
+        edit_int64(now - retention, ed1));
+   db_sql_query(ua->db, query.c_str(), NULL, NULL);
+   db_unlock(ua->db);
+
+   ua->info_msg(_("Pruned Jobs from JobHisto catalog.\n"));
+
+   return true;
+}
+
 /*
  * Prune File records from the database. For any Job which
  * is older than the retention period, we unconditionally delete
@@ -179,8 +234,10 @@ int prunecmd(UAContext *ua, const char *cmd)
  *
  * This routine assumes you want the pruning to be done. All checking
  *  must be done before calling this routine.
+ *
+ * Note: pool can possibly be NULL.
  */
-int prune_files(UAContext *ua, CLIENT *client)
+int prune_files(UAContext *ua, CLIENT *client, POOL *pool)
 {
    struct del_ctx del;
    struct s_count_ctx cnt;
@@ -192,18 +249,25 @@ int prune_files(UAContext *ua, CLIENT *client)
    db_lock(ua->db);
    memset(&cr, 0, sizeof(cr));
    memset(&del, 0, sizeof(del));
-   bstrncpy(cr.Name, client->hdr.name, sizeof(cr.Name));
+   bstrncpy(cr.Name, client->name(), sizeof(cr.Name));
    if (!db_create_client_record(ua->jcr, ua->db, &cr)) {
       db_unlock(ua->db);
       return 0;
    }
 
-   period = client->FileRetention;
+   if (pool && pool->FileRetention > 0) {
+      period = pool->FileRetention;
+   } else {
+      period = client->FileRetention;
+   }
    now = (utime_t)time(NULL);
 
-   /* Select Jobs -- for counting */
-   Mmsg(query, count_select_job, edit_int64(now - period, ed1), 
-        edit_int64(cr.ClientId, ed2));
+//   edit_utime(now-period, ed1, sizeof(ed1));
+//   Jmsg(ua->jcr, M_INFO, 0, _("Begin pruning Jobs older than %s secs.\n"), ed1);
+   Jmsg(ua->jcr, M_INFO, 0, _("Begin pruning Jobs.\n"));
+   /* Select Jobs -- for counting */ 
+   edit_int64(now - period, ed1);
+   Mmsg(query, count_select_job, ed1, edit_int64(cr.ClientId, ed2));
    Dmsg3(050, "select now=%u period=%u sql=%s\n", (uint32_t)now, 
                (uint32_t)period, query.c_str());
    cnt.count = 0;
@@ -259,14 +323,16 @@ static void drop_temp_tables(UAContext *ua)
 
 static bool create_temp_tables(UAContext *ua)
 {
-   int i;
    /* Create temp tables and indicies */
-   for (i=0; create_deltabs[i]; i++) {
-      if (!db_sql_query(ua->db, create_deltabs[i], NULL, (void *)NULL)) {
-         ua->error_msg("%s", db_strerror(ua->db));
-         Dmsg0(050, "create DelTables table failed\n");
-         return false;
-      }
+   if (!db_sql_query(ua->db, create_deltabs[db_type], NULL, (void *)NULL)) {
+      ua->error_msg("%s", db_strerror(ua->db));
+      Dmsg0(050, "create DelTables table failed\n");
+      return false;
+   }
+   if (!db_sql_query(ua->db, create_delindex, NULL, (void *)NULL)) {
+       ua->error_msg("%s", db_strerror(ua->db));
+       Dmsg0(050, "create DelInx1 index failed\n");
+       return false;
    }
    return true;
 }
@@ -285,7 +351,7 @@ static bool create_temp_tables(UAContext *ua)
  *
  * For Restore Jobs there are no restrictions.
  */
-int prune_jobs(UAContext *ua, CLIENT *client, int JobType)
+int prune_jobs(UAContext *ua, CLIENT *client, POOL *pool, int JobType)
 {
    struct del_ctx del;
    POOL_MEM query(PM_MESSAGE);
@@ -303,7 +369,11 @@ int prune_jobs(UAContext *ua, CLIENT *client, int JobType)
       return 0;
    }
 
-   period = client->JobRetention;
+   if (pool && pool->JobRetention > 0) {
+      period = pool->JobRetention;
+   } else {
+      period = client->JobRetention;
+   }
    now = (utime_t)time(NULL);
 
    /* Drop any previous temporary tables still there */
@@ -314,10 +384,13 @@ int prune_jobs(UAContext *ua, CLIENT *client, int JobType)
       goto bail_out;
    }
 
+
    /*
     * Select all files that are older than the JobRetention period
     *  and stuff them into the "DeletionCandidates" table.
     */
+   edit_utime(period, ed1, sizeof(ed1));
+   Jmsg(ua->jcr, M_INFO, 0, _("Begin pruning Jobs older than %s.\n"), ed1);
    edit_int64(now - period, ed1);
    Mmsg(query, insert_delcand, (char)JobType, ed1, 
         edit_int64(cr.ClientId, ed2));
@@ -348,6 +421,9 @@ int prune_jobs(UAContext *ua, CLIENT *client, int JobType)
    case JT_ADMIN:
       Mmsg(query, select_admin_del, ed1, ed2);
       break;
+   case JT_COPY:
+      Mmsg(query, select_copy_del, ed1, ed2);
+      break;
    case JT_MIGRATE:
       Mmsg(query, select_migrate_del, ed1, ed2);
       break;
@@ -425,11 +501,8 @@ int get_prune_list_for_volume(UAContext *ua, MEDIA_DBR *mr, del_ctx *del)
 {
    POOL_MEM query(PM_MESSAGE);
    int count = 0;
-   int i;          
    utime_t now, period;
    char ed1[50], ed2[50];
-   JCR *jcr;
-   bool skip;
 
    if (mr->Enabled == 2) {
       return 0;                    /* cannot prune Archived volumes */
@@ -454,26 +527,44 @@ int get_prune_list_for_volume(UAContext *ua, MEDIA_DBR *mr, del_ctx *del)
       Dmsg0(050, "Count failed\n");
       goto bail_out;
    }
+   count = exclude_running_jobs_from_list(del);
+   
+bail_out:
+   return count;
+}
+
+/*
+ * We have a list of jobs to prune or purge. If any of them is
+ *   currently running, we set its JobId to zero which effectively
+ *   excludes it.
+ *
+ * Returns the number of jobs that can be prunned or purged.
+ *
+ */
+int exclude_running_jobs_from_list(del_ctx *prune_list)
+{
+   int count = 0;
+   JCR *jcr;
+   bool skip;
+   int i;          
 
    /* Do not prune any job currently running */
-   for (i=0; i < del->num_ids; i++) {
+   for (i=0; i < prune_list->num_ids; i++) {
       skip = false;
       foreach_jcr(jcr) {
-         if (jcr->JobId == del->JobId[i]) {
-            Dmsg2(150, "skip same job JobId[%d]=%d\n", i, (int)del->JobId[i]);
-            del->JobId[i] = 0;
+         if (jcr->JobId == prune_list->JobId[i]) {
+            Dmsg2(050, "skip running job JobId[%d]=%d\n", i, (int)prune_list->JobId[i]);
+            prune_list->JobId[i] = 0;
             skip = true;
             break;
          }
       }
       endeach_jcr(jcr);
       if (skip) {
-         continue;
+         continue;  /* don't increment count */
       }
-      Dmsg2(150, "accept JobId[%d]=%d\n", i, (int)del->JobId[i]);
+      Dmsg2(050, "accept JobId[%d]=%d\n", i, (int)prune_list->JobId[i]);
       count++;
    }
-
-bail_out:
    return count;
 }
