@@ -4,7 +4,7 @@
  *
  *     Kern Sibbald, December MMI
  *
- *   Version $Id: ua_query.c 5713 2007-10-03 11:36:47Z kerns $
+ *   Version $Id$
  */
 /*
    Bacula® - The Network Backup Solution
@@ -28,7 +28,7 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 
-   Bacula® is a registered trademark of John Walker.
+   Bacula® is a registered trademark of Kern Sibbald.
    The licensor of Bacula is the Free Software Foundation Europe
    (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
    Switzerland, email:ftf@fsfeurope.org.
