@@ -20,7 +20,7 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 
-   Bacula® is a registered trademark of John Walker.
+   Bacula® is a registered trademark of Kern Sibbald.
    The licensor of Bacula is the Free Software Foundation Europe
    (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
    Switzerland, email:ftf@fsfeurope.org.
@@ -31,7 +31,7 @@
  *
  * Kern Sibbald, March MMV
  *
- *   Version $Id: pythonfd.c 7297 2008-07-03 10:50:08Z kerns $
+ *   Version $Id$
  *
  */
 
@@ -42,10 +42,7 @@
 #undef _POSIX_C_SOURCE
 #include <Python.h>
 
-
-extern JCR *get_jcr_from_PyObject(PyObject *self);
-extern PyObject *find_method(PyObject *eventsObject, PyObject *method, 
-          const char *name);
+#include <lib/pythonlib.h>
 
 /* Forward referenced functions */
 static PyObject *set_job_events(PyObject *self, PyObject *arg);
@@ -111,9 +108,9 @@ PyObject *job_getattr(PyObject *self, char *attrname)
    case 0:                            /* FD's name */
       return Py_BuildValue((char *)getvars[i].fmt, my_name);
    case 1:                            /* level */
-      return Py_BuildValue((char *)getvars[i].fmt, job_level_to_str(jcr->JobLevel));
+      return Py_BuildValue((char *)getvars[i].fmt, job_level_to_str(jcr->getJobLevel()));
    case 2:                            /* type */
-      return Py_BuildValue((char *)getvars[i].fmt, job_type_to_str(jcr->JobType));
+      return Py_BuildValue((char *)getvars[i].fmt, job_type_to_str(jcr->getJobType()));
    case 3:                            /* JobId */
       return Py_BuildValue((char *)getvars[i].fmt, jcr->JobId);
    case 4:                            /* Client */
