@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2003-2007 Free Software Foundation Europe e.V.
+   Copyright (C) 2003-2009 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -20,7 +20,7 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 
-   Bacula® is a registered trademark of John Walker.
+   Bacula® is a registered trademark of Kern Sibbald.
    The licensor of Bacula is the Free Software Foundation Europe
    (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
    Switzerland, email:ftf@fsfeurope.org.
@@ -30,7 +30,7 @@
  *
  *    Kern Sibbald, May MMIII
  *
- *   Version $Id: enable_priv.c 5063 2007-06-22 17:21:58Z kerns $
+ *   Version $Id$
  *
  */
 
@@ -63,10 +63,10 @@ int enable_backup_privileges(JCR *jcr, int ignore_errors)
 
 #if defined(HAVE_WIN32)
 
-void win_error(JCR *jcr, char *prefix, DWORD lerror);
+void win_error(JCR *jcr, const char *prefix, DWORD lerror);
 
 static int
-enable_priv(JCR *jcr, HANDLE hToken, char *name, int ignore_errors)
+enable_priv(JCR *jcr, HANDLE hToken, const char *name, int ignore_errors)
 {
     TOKEN_PRIVILEGES tkp;
     DWORD lerror;
