@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2000-2008 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2009 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -20,7 +20,7 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 
-   Bacula® is a registered trademark of John Walker.
+   Bacula® is a registered trademark of Kern Sibbald.
    The licensor of Bacula is the Free Software Foundation Europe
    (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
    Switzerland, email:ftf@fsfeurope.org.
@@ -29,7 +29,7 @@
  * Append code for Storage daemon
  *  Kern Sibbald, May MM
  *
- *  Version $Id: append.c 6831 2008-04-16 09:49:47Z kerns $
+ *  Version $Id$
  */
 
 #include "bacula.h"
@@ -50,8 +50,7 @@ bool do_append_data(JCR *jcr)
 {
    int32_t n;
    int32_t file_index, stream, last_file_index;
-   BSOCK *ds;
-   BSOCK *fd_sock = jcr->file_bsock;
+   BSOCK *fd = jcr->file_bsock;
    bool ok = true;
    DEV_RECORD rec;
    char buf1[100], buf2[100];
@@ -74,9 +73,7 @@ bool do_append_data(JCR *jcr)
 
    memset(&rec, 0, sizeof(rec));
 
-   ds = fd_sock;
-
-   if (!ds->set_buffer_size(dcr->device->max_network_buffer_size, BNET_SETBUF_WRITE)) {
+   if (!fd->set_buffer_size(dcr->device->max_network_buffer_size, BNET_SETBUF_WRITE)) {
       set_jcr_job_status(jcr, JS_ErrorTerminated);
       Jmsg0(jcr, M_FATAL, 0, _("Unable to set network buffer size.\n"));
       return false;
@@ -116,10 +113,10 @@ bool do_append_data(JCR *jcr)
    }
 
    /* Tell File daemon to send data */
-   if (!fd_sock->fsend(OK_data)) {
+   if (!fd->fsend(OK_data)) {
       berrno be;
       Jmsg1(jcr, M_FATAL, 0, _("Network send error to FD. ERR=%s\n"),
-            be.bstrerror(fd_sock->b_errno));
+            be.bstrerror(fd->b_errno));
       ok = false;
    }
 
@@ -148,40 +145,24 @@ bool do_append_data(JCR *jcr)
        *  The stream header consists of the following:
        *    file_index (sequential Bacula file index, base 1)
        *    stream     (Bacula number to distinguish parts of data)
-       *    info       (Info for Storage daemon -- compressed, encryped, ...)
+       *    info       (Info for Storage daemon -- compressed, encrypted, ...)
        *       info is not currently used, so is read, but ignored!
        */
-     if ((n=bget_msg(ds)) <= 0) {
-         if (n == BNET_SIGNAL && ds->msglen == BNET_EOD) {
+     if ((n=bget_msg(fd)) <= 0) {
+         if (n == BNET_SIGNAL && fd->msglen == BNET_EOD) {
             break;                    /* end of data */
          }
          Jmsg1(jcr, M_FATAL, 0, _("Error reading data header from FD. ERR=%s\n"),
-               ds->bstrerror());
+               fd->bstrerror());
          ok = false;
          break;
       }
 
-      /*
-       * This hand scanning is a bit more complicated than a simple
-       *   sscanf, but it allows us to handle any size integer up to
-       *   int64_t without worrying about whether %d, %ld, %lld, or %q
-       *   is the correct format for each different architecture.
-       * It is a real pity that sscanf() is not portable.
-       */
-      char *p = ds->msg;
-      while (B_ISSPACE(*p)) {
-         p++;
-      }
-      file_index = (int32_t)str_to_int64(p);
-      while (B_ISDIGIT(*p)) {
-         p++;
-      }
-      if (!B_ISSPACE(*p) || !B_ISDIGIT(*(p+1))) {
-         Jmsg1(jcr, M_FATAL, 0, _("Malformed data header from FD: %s\n"), ds->msg);
+      if (sscanf(fd->msg, "%ld %ld", &file_index, &stream) != 2) {
+         Jmsg1(jcr, M_FATAL, 0, _("Malformed data header from FD: %s\n"), fd->msg);
          ok = false;
          break;
       }
-      stream = (int32_t)str_to_int64(p);
 
       Dmsg2(890, "<filed: Header FilInx=%d stream=%d\n", file_index, stream);
 
@@ -199,13 +180,13 @@ bool do_append_data(JCR *jcr)
       /* Read data stream from the File daemon.
        *  The data stream is just raw bytes
        */
-      while ((n=bget_msg(ds)) > 0 && !job_canceled(jcr)) {
+      while ((n=bget_msg(fd)) > 0 && !job_canceled(jcr)) {
          rec.VolSessionId = jcr->VolSessionId;
          rec.VolSessionTime = jcr->VolSessionTime;
          rec.FileIndex = file_index;
          rec.Stream = stream;
-         rec.data_len = ds->msglen;
-         rec.data = ds->msg;            /* use message buffer */
+         rec.data_len = fd->msglen;
+         rec.data = fd->msg;            /* use message buffer */
 
          Dmsg4(850, "before writ_rec FI=%d SessId=%d Strm=%s len=%d\n",
             rec.FileIndex, rec.VolSessionId, 
@@ -235,28 +216,31 @@ bool do_append_data(JCR *jcr)
          if (stream == STREAM_UNIX_ATTRIBUTES || stream == STREAM_UNIX_ATTRIBUTES_EX ||
              crypto_digest_stream_type(stream) != CRYPTO_DIGEST_NONE) {
             if (!jcr->no_attributes) {
+               BSOCK *dir = jcr->dir_bsock;
                if (are_attributes_spooled(jcr)) {
-                  jcr->dir_bsock->set_spooling();
+                  dir->set_spooling();
                }
                Dmsg0(850, "Send attributes to dir.\n");
                if (!dir_update_file_attributes(dcr, &rec)) {
-                  jcr->dir_bsock->clear_spooling();
+                  dir->clear_spooling();
                   Jmsg(jcr, M_FATAL, 0, _("Error updating file attributes. ERR=%s\n"),
-                     jcr->dir_bsock->bstrerror());
+                     dir->bstrerror());
                   ok = false;
                   break;
                }
-               jcr->dir_bsock->clear_spooling();
+               dir->clear_spooling();
             }
          }
          Dmsg0(650, "Enter bnet_get\n");
       }
       Dmsg1(650, "End read loop with FD. Stat=%d\n", n);
 
-      if (is_bnet_error(ds)) {
-         Dmsg1(350, "Network read error from FD. ERR=%s\n", ds->bstrerror());
-         Jmsg1(jcr, M_FATAL, 0, _("Network error on data channel. ERR=%s\n"),
-               ds->bstrerror());
+      if (fd->is_error()) {
+         if (!job_canceled(jcr)) {
+            Dmsg1(350, "Network read error from FD. ERR=%s\n", fd->bstrerror());
+            Jmsg1(jcr, M_FATAL, 0, _("Network error reading from FD. ERR=%s\n"),
+                  fd->bstrerror());
+         }
          ok = false;
          break;
       }
@@ -265,18 +249,25 @@ bool do_append_data(JCR *jcr)
    /* Create Job status for end of session label */
    set_jcr_job_status(jcr, ok?JS_Terminated:JS_ErrorTerminated);
 
-   /* Terminate connection with FD */
-   ds->fsend(OK_append);
-   do_fd_commands(jcr);               /* finish dialog with FD */
+   if (ok) {
+      /* Terminate connection with FD */
+      fd->fsend(OK_append);
+      do_fd_commands(jcr);               /* finish dialog with FD */
+   } else {
+      fd->fsend("3999 Failed append\n");
+   }
 
-
-   time_t job_elapsed = time(NULL) - jcr->run_time;
+   /*
+    * Don't use time_t for job_elapsed as time_t can be 32 or 64 bits,
+    *   and the subsequent Jmsg() editing will break
+    */
+   int32_t job_elapsed = time(NULL) - jcr->run_time;
 
    if (job_elapsed <= 0) {
       job_elapsed = 1;
    }
 
-   Jmsg(dcr->jcr, M_INFO, 0, _("Job write elapsed time = %02d:%02d:%02d, Transfer rate = %s bytes/second\n"),
+   Jmsg(dcr->jcr, M_INFO, 0, _("Job write elapsed time = %02d:%02d:%02d, Transfer rate = %s Bytes/second\n"),
          job_elapsed / 3600, job_elapsed % 3600 / 60, job_elapsed % 60,
          edit_uint64_with_suffix(jcr->JobBytes / job_elapsed, ec));
 
@@ -289,8 +280,11 @@ bool do_append_data(JCR *jcr)
     */
    if (ok || dev->can_write()) {
       if (!write_session_label(dcr, EOS_LABEL)) {
-         Jmsg1(jcr, M_FATAL, 0, _("Error writting end session label. ERR=%s\n"),
-               dev->bstrerror());
+         /* Print only if ok and not cancelled to avoid spurious messages */
+         if (ok && !job_canceled(jcr)) {
+            Jmsg1(jcr, M_FATAL, 0, _("Error writing end session label. ERR=%s\n"),
+                  dev->bstrerror());
+         }
          set_jcr_job_status(jcr, JS_ErrorTerminated);
          ok = false;
       }
@@ -301,9 +295,13 @@ bool do_append_data(JCR *jcr)
       Dmsg0(90, "back from write_end_session_label()\n");
       /* Flush out final partial block of this session */
       if (!write_block_to_device(dcr)) {
-         Jmsg2(jcr, M_FATAL, 0, _("Fatal append error on device %s: ERR=%s\n"),
-               dev->print_name(), dev->bstrerror());
-         Dmsg0(100, _("Set ok=FALSE after write_block_to_device.\n"));
+         /* Print only if ok and not cancelled to avoid spurious messages */
+         if (ok && !job_canceled(jcr)) {
+            Jmsg2(jcr, M_FATAL, 0, _("Fatal append error on device %s: ERR=%s\n"),
+                  dev->print_name(), dev->bstrerror());
+            Dmsg0(100, _("Set ok=FALSE after write_block_to_device.\n"));
+         }
+         set_jcr_job_status(jcr, JS_ErrorTerminated);
          ok = false;
       }
       if (dev->VolCatInfo.VolCatName[0] == 0) {
@@ -316,7 +314,7 @@ bool do_append_data(JCR *jcr)
    if (!ok) {
       discard_data_spool(dcr);
    } else {
-      /* Note: if commit is OK, the device will remain locked */
+      /* Note: if commit is OK, the device will remain blocked */
       commit_data_spool(dcr);
    }
 
