@@ -1,19 +1,16 @@
-/*
- *  Version $Id: version.h 8276 2009-01-03 08:38:38Z kerns $
- */
 
 #undef  VERSION
-#define VERSION "2.4.4"
-#define BDATE   "28 December 2008"
-#define LSMDATE "28Dec08"
+#define VERSION "5.0.1"
+#define BDATE   "24 February 2010"
+#define LSMDATE "24Feb10"
 
-#define PROG_COPYRIGHT "Copyright (C) %d-2008 Free Software Foundation Europe e.V.\n"
-#define BYEAR "2008"       /* year for copyright messages in progs */
+#define PROG_COPYRIGHT "Copyright (C) %d-2010 Free Software Foundation Europe e.V.\n"
+#define BYEAR "2010"       /* year for copyright messages in progs */
 
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2000-2008 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2010 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -38,6 +35,28 @@
    Switzerland, email:ftf@fsfeurope.org.
 */
 
+/* Shared object library versions */
+#define LIBBAC_LT_CURRENT  5
+#define LIBBAC_LT_REVISION 0
+#define LIBBAC_LT_AGE 1
+
+#define LIBBACCFG_LT_CURRENT 5
+#define LIBBACCFG_LT_REVISION 0
+#define LIBBACCFG_LT_AGE 1
+
+#define LIBBACPY_LT_CURRENT 5
+#define LIBBACPY_LT_REVISION 0
+#define LIBBACPY_LT_AGE 1
+
+#define LIBBACSQL_LT_CURRENT 5
+#define LIBBACSQL_LT_REVISION 0
+#define LIBBACSQL_LT_AGE 1
+
+#define LIBBACFIND_LT_CURRENT 5
+#define LIBBACFIND_LT_REVISION 0
+#define LIBBACFIND_LT_AGE 1
+
+
 
 /* Debug flags */
 #undef  DEBUG
@@ -54,18 +73,50 @@
  *  penalties.
  */
 #ifdef DEVELOPER
-#define SMCHECK
+# define SMCHECK
+#endif
+
+/*
+ * _USE_LOCKMGR does lock/unlock mutex tracking (dead lock)
+ *   it can always be turned on, but we advise to use it only
+ *   for debug
+ */
+#if DEVELOPER
+# ifndef _USE_LOCKMGR
+#  define _USE_LOCKMGR
+# endif /* _USE_LOCKMGR */
+/*
+ * Enable priority management with the lock manager
+ *
+ * Note, turning this on will cause the Bacula SD to abort if
+ *  mutexes are executed out of order, which could lead to a
+ *  deadlock.  However, note that this is not necessarily a
+ *  deadlock, so turn this on only for debugging.
+ */
+#define USE_LOCKMGR_PRIORITY
+#endif  /* DEVELOPER */
+
+#if !HAVE_LINUX_OS && !HAVE_SUN_OS && !HAVE_DARWIN_OS && !HAVE_FREEBSD_OS
+# undef _USE_LOCKMGR
+#endif
+
+/*
+ * USE_VTAPE is a dummy tape driver. This is useful to
+ *  run regress test.
+ */
+#ifdef HAVE_LINUX_OS
+# define USE_VTAPE
 #endif
 
 /* 
  * for fastest speed but you must have a UPS to avoid unwanted shutdowns
  */
-#define SQLITE3_INIT_QUERY "PRAGMA synchronous = OFF"
+//#define SQLITE3_INIT_QUERY "PRAGMA synchronous = OFF"
 
 /*
  * for more safety, but is 30 times slower than above
  */
-//#define SQLITE3_INIT_QUERY "PRAGMA synchronous = NORMAL"
+#define SQLITE3_INIT_QUERY "PRAGMA synchronous = NORMAL"
 
 /*
  * This should always be on. It enables data encryption code 
