@@ -5,7 +5,7 @@
  *
  *    Nicolas Boichat, April 2004
  *
- *    Version $Id: wxbtreectrl.h 4992 2007-06-07 14:46:43Z kerns $
+ *    Version $Id$
  */
 /*
    Bacula® - The Network Backup Solution
@@ -29,7 +29,7 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 
-   Bacula® is a registered trademark of John Walker.
+   Bacula® is a registered trademark of Kern Sibbald.
    The licensor of Bacula is the Free Software Foundation Europe
    (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
    Switzerland, email:ftf@fsfeurope.org.
